import { matches, Validator } from '@etherealengine/engine/src/common/functions/MatchesUtils'
import {
  defineAction,
  defineState,
  getMutableState,
  syncStateWithLocalStorage,
  useState
} from '@etherealengine/hyperflux'

export const AppState = defineState({
  name: 'AppState',
  initial: () => ({
    showTopShelf: true,
    showBottomShelf: true,
    showTouchPad: true
  }),
  onCreate: () => {
    syncStateWithLocalStorage(AppState, ['showTopShelf', 'showBottomShelf'])
  }
})

export const AppServiceReceptor = (action) => {
  const s = getMutableState(AppState)
  matches(action)
    .when(AppAction.showTopShelf.matches, (action) => {
      return s.showTopShelf.set(action.show)
    })
    .when(AppAction.showBottomShelf.matches, (action) => {
      return s.showBottomShelf.set(action.show)
    })
    .when(AppAction.showTouchPad.matches, (action) => {
      return s.showTouchPad.set(action.show)
    })
}

export class AppAction {
  static showTopShelf = defineAction({
    type: 'ee.client.App.SHOW_TOP_SHELF' as const,
    show: matches.boolean
  })
  static showBottomShelf = defineAction({
    type: 'ee.client.App.SHOW_BOTTOM_SHELF' as const,
    show: matches.boolean
  })
  static showTouchPad = defineAction({
    type: 'ee.client.App.SHOW_TOUCH_PAD' as const,
    show: matches.boolean
  })
}
